#include "rectangle.h"

int isAreaOccupied(Rectangle* rects, int numRects, int x, int y, int l, int h) {
    int i;
    int newRectRight = x + l;
    int newRectTop = y + h;

    if (newRectRight > TABLE_WIDTH || newRectTop > TABLE_HEIGHT || x < 1 || y < 1) {
        return 1;
    }

    for (i = 0; i < numRects; i++) {
        Rectangle* rect = &rects[i];
        int rectRight = rect->x + rect->l;
        int rectTop = rect->y + rect->h;

        if (rect->x < newRectRight && rectRight > x && rect->y < newRectTop && rectTop > y) {
            return 1;
        }
    }

    return 0;
}

void createRectangle(Rectangle* rects, int* numRects, int x, int y, int l, int h) {
    if (*numRects >= MAX_RECTANGLES) {
        printf("Error: Maximum number of rectangles reached.\n");
        return;
    }

    if (isAreaOccupied(rects, *numRects, x, y, l, h)) {
        printf("Error: The specified area is already occupied or exceeds table boundaries.\n");
    } else {
        Rectangle* rect = &rects[*numRects];
        rect->x = x;
        rect->y = y;
        rect->l = l;
        rect->h = h;
        (*numRects)++;
    }
}

void moveSide(Rectangle* rects, int numRects, int x, int y, int p) {
    int moved = 0;

    for (int i = 0; i < numRects; i++) {
        Rectangle* rect = &rects[i];
        if (x >= rect->x && x < rect->x + rect->l && y >= rect->y && y < rect->y + rect->h) {
            int newX = rect->x + p;

            if (newX < 1 || newX + rect->l > TABLE_WIDTH) {
                printf("Error: Rectangle cannot be moved due to table boundaries.\n");
                return;
            }

            int overlap = 0;
            for (int j = 0; j < numRects; j++) {
                if (j != i) {
                    Rectangle* otherRect = &rects[j];
                    if (newX < otherRect->x + otherRect->l && newX + rect->l > otherRect->x &&
                        rect->y < otherRect->y + otherRect->h && rect->y + rect->h > otherRect->y) {
                        overlap = 1;
                        break;
                    }
                }
            }

            if (!overlap) {
                rect->x = newX;
                moved = 1;
            } else {
                printf("Error: Rectangle cannot be moved due to overlap with another rectangle.\n");
            }

            break;
        }
    }

    if (!moved) {
        printf("Error: No rectangle found with the specified coordinates or cannot be moved.\n");
    }
}

void applyGravity(Rectangle* rects, int numRects) {
    int settled = 0;

    while (!settled) {
        settled = 1;

        for (int i = 0; i < numRects; i++) {
            Rectangle* rect = &rects[i];
            int newY = rect->y;

            for (int y = rect->y - 1; y >= 1; y--) {
                int canFall = 1;
                for (int j = 0; j < numRects; j++) {
                    if (i != j) {
                        Rectangle* otherRect = &rects[j];
                        if (rect->x < otherRect->x + otherRect->l && rect->x + rect->l > otherRect->x &&
                            y < otherRect->y + otherRect->h && y + rect->h > otherRect->y) {
                            canFall = 0;
                            break;
                        }
                    }
                }

                if (canFall) {
                    newY = y;
                } else {
                    break;
                }
            }

            if (newY != rect->y) {
                rect->y = newY;
                settled = 0;
            }
        }
    }
}
