#include <stdio.h>
#include "rectangle.h"
#include "list.h"

void createRectangle(Node** head, int x, int y, int l, int h) {
    if (isAreaOccupied(*head, x, y, l, h)) {
        printf("Error: The specified area is already occupied or exceeds table boundaries.\n");
    } else {
        addNode(head, x, y, l, h);
        printf("Success: Rectangle created successfully.\n");
    }
}

void moveSide(Node* head, int x, int y, int p) {
    Node* temp = head;
    while (temp != NULL) {
        Rectangle* rect = &temp->rect;
        if (x >= rect->x && x < rect->x + rect->l && y >= rect->y && y < rect->y + rect->h) {
            int newX = rect->x + p;

            if (newX < 1 || newX + rect->l > TABLE_WIDTH) {
                printf("Error: Rectangle cannot be moved due to table boundaries.\n");
                return;
            }

            int overlap = 0;
            Node* other = head;
            while (other != NULL) {
                if (other != temp) {
                    Rectangle* otherRect = &other->rect;
                    if (newX < otherRect->x + otherRect->l && newX + rect->l > otherRect->x &&
                        rect->y < otherRect->y + otherRect->h && rect->y + rect->h > otherRect->y) {
                        overlap = 1;
                        break;
                    }
                }
                other = other->next;
            }

            if (!overlap) {
                rect->x = newX;
                printf("Success: Rectangle moved successfully.\n");
            } else {
                printf("Error: Rectangle cannot be moved due to overlap with another rectangle.\n");
            }
            return;
        }
        temp = temp->next;
    }
    printf("Error: No rectangle found with the specified coordinates or cannot be moved.\n");
}

void applyGravity(Node* head) {
    int settled = 0;
    while (!settled) {
        settled = 1;
        Node* temp = head;
        while (temp != NULL) {
            Rectangle* rect = &temp->rect;
            int newY = rect->y;

            for (int y = rect->y - 1; y >= 1; y--) {
                int canFall = 1;
                Node* other = head;
                while (other != NULL) {
                    if (other != temp) {
                        Rectangle* otherRect = &other->rect;
                        if (rect->x < otherRect->x + otherRect->l && rect->x + rect->l > otherRect->x &&
                            y < otherRect->y + otherRect->h && y + rect->h > otherRect->y) {
                            canFall = 0;
                            break;
                        }
                    }
                    other = other->next;
                }
                if (canFall) {
                    newY = y;
                } else {
                    break;
                }
            }

            if (newY != rect->y) {
                rect->y = newY;
                settled = 0;
            }
            temp = temp->next;
        }
    }
}

int mergePossivel(Rectangle* rect1, Rectangle* rect2) {
    return (rect1->x == rect2->x && rect1->l == rect2->l && 
            (rect1->y == rect2->y + rect2->h || rect2->y == rect1->y + rect1->h));
}

void mergeRetangulos(Rectangle* rect1, Rectangle* rect2) {
    if (rect1->y > rect2->y) {
        rect1->y = rect2->y;
    }
    rect1->h += rect2->h;
}

void mergeCommand(Node** head, int x1, int y1, int x2, int y2) {
    Node* temp = *head;
    Node* rect1Node = NULL;
    Node* rect2Node = NULL;

    // Encontra os nós que contêm os pontos (x1, y1) e (x2, y2)
    while (temp != NULL) {
        if (rect1Node == NULL && x1 >= temp->rect.x && x1 < temp->rect.x + temp->rect.l &&
            y1 >= temp->rect.y && y1 < temp->rect.y + temp->rect.h) {
            rect1Node = temp;
        }
        if (rect2Node == NULL && x2 >= temp->rect.x && x2 < temp->rect.x + temp->rect.l &&
            y2 >= temp->rect.y && y2 < temp->rect.y + temp->rect.h) {
            rect2Node = temp;
        }
        temp = temp->next;
    }

    if (rect1Node != NULL && rect2Node != NULL && mergePossivel(&rect1Node->rect, &rect2Node->rect)) {
        mergeRetangulos(&rect1Node->rect, &rect2Node->rect);
        removeNode(head, rect2Node);
        printf("Success: Rectangles merged successfully.\n");
    } else {
        printf("Error: Cannot merge the specified rectangles.\n");
    }
}

void alertaSobreposicao(Node* head) {
    Node* temp = head;
    while (temp != NULL) {
        Node* other = temp->next;
        while (other != NULL) {
            if (mergePossivel(&temp->rect, &other->rect)) {
                printf("Alerta: Os retângulos em (%d, %d) e (%d, %d) podem ser unidos.\n", 
                       temp->rect.x, temp->rect.y, other->rect.x, other->rect.y);
            }
            other = other->next;
        }
        temp = temp->next;
    }
}

void printRectangle(Node* head) {
    // Desenha as linhas da tabela
    for (int i = TABLE_HEIGHT; i >= 0; i--) {
        // Imprime o número da linha no lado esquerdo (inverso)
        printf("%02d |", i);

        // Imprime as células da tabela
        for (int j = 0; j < TABLE_WIDTH; j++) {
            int isInsideRect = 0;
            int isOnBorder = 0;

            Node* temp = head;
            while (temp != NULL) {
                Rectangle* rect = &temp->rect;

                // Verifica se o ponto (j, i) está dentro do retângulo
                if ((j >= rect->x && j < rect->x + rect->l) && (i >= rect->y && i < rect->y + rect->h)) {
                    isInsideRect = 1;
                }

                // Verifica se a linha i está na borda do retângulo (horizontal)
                if (i == rect->y || i == rect->y + rect->h - 1) {
                    if (j >= rect->x && j < rect->x + rect->l) {
                        isOnBorder = 1;
                    }
                }

                // Verifica se a coluna j está na borda do retângulo (vertical)
                if (j == rect->x || j == rect->x + rect->l - 1) {
                    if (i >= rect->y && i < rect->y + rect->h) {
                        isOnBorder = 1;
                    }
                }
                temp = temp->next;
            }

            if (isOnBorder)
                printf("x");
            else if (isInsideRect)
                printf("o");
            else
                printf(" ");
        }
        printf("\n");
    }

    // Imprime a linha inferior da tabela
    printf("   *");
    for (int i = 0; i <= TABLE_WIDTH; i++) {
        printf("-");
    }
    printf("\n");

    // Desenha a linha da largura
    printf("    0");
    for (int i = 1; i <= TABLE_WIDTH / 10; i++) {
        printf("         %d", i); // Imprime os números das dezenas
    }
    printf("\n");

    printf("    0123456789");
    for (int i = 0; i < (TABLE_WIDTH / 10) - 1; i++) {
        printf("0123456789"); // Imprime os números das unidades
    }
    printf("0");
    printf("\n");
}
